package org.sf.charttaglibs.charts;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import javax.servlet.jsp.JspException;

import org.apache.commons.beanutils.BeanUtils;

import org.jfree.chart.ChartRenderingInfo;
import org.jfree.data.category.DefaultCategoryDataset;

import org.sf.charttaglibs.BaseChartTag;

/**
 * Abstract class theat implement the common method of all category based chart
 * @author mmohamed |at| dcc |dot| uchile |dot| cl
 *
 */
public abstract class CategoryBasedChart extends BaseChartTag {
	protected String collection = null;
	protected String category = null;
	protected String serie = null;
	protected String value = null;
	protected String domainAxisLabel = null;
	protected String rangeAxisLabel = null;
	protected String plotOrientation = null;
	protected boolean threeD = false;
	
	/**
	 * Getting the data set from the page
	 * @return DefaultCategoryDataset from the bar chart
	 * @throws JspException if the category, serie or value are not present in the bean
	 */
	protected DefaultCategoryDataset getDataSet() throws JspException {
		
		String filename = null;
		ChartRenderingInfo info = null;
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		if (collection != null) {
			Iterator collIterator = getIterator(collection, null);
			while (collIterator.hasNext()) {
				
				Object bean = collIterator.next();
				Object cat = null;
				Object ser = serie;
				Object val = null;
				
				try {
					cat = BeanUtils.getProperty(bean, category);
					if (cat == null) {
						cat = "";
					}
				} catch (IllegalAccessException e) {
					throw new JspException(
							messages.getMessage("getter.access", category, collection));
				} catch (InvocationTargetException e) {
					Throwable t = e.getTargetException();
					throw new JspException(
							messages.getMessage("getter.result", category, t.toString()));
				} catch (NoSuchMethodException e) {
					throw new JspException(
							messages.getMessage("getter.method", category, collection));
				}

				try {
					if (serie != null) {
						ser = BeanUtils.getProperty(bean, serie);
					}
					
					if (ser == null) {
						ser = "";
					}
				} catch (IllegalAccessException e) {
					throw new JspException(
							messages.getMessage("getter.access", ser, collection));
				} catch (InvocationTargetException e) {
					Throwable t = e.getTargetException();
					throw new JspException(
							messages.getMessage("getter.result", ser, t.toString()));
				} catch (NoSuchMethodException e) {
					throw new JspException(
							messages.getMessage("getter.method", ser, collection));
				}
				
				try {
					if (value != null) {
						val = BeanUtils.getProperty(bean, value);
					}
					
					if (val == null) {
						val = "";
					}
				} catch (IllegalAccessException e) {
					throw new JspException(
							messages.getMessage("getter.access", cat, collection));
				} catch (InvocationTargetException e) {
					Throwable t = e.getTargetException();
					throw new JspException(
							messages.getMessage("getter.result", cat, t.toString()));
				} catch (NoSuchMethodException e) {
					throw new JspException(
							messages.getMessage("getter.method", cat, collection));
				}
				
				dataset.setValue( new Float(val.toString()), ser.toString(), cat.toString());
			}
		}
		return dataset;
	}

    /**
     * Release any acquired resources.
     */
	public void release() {
		super.release();
		collection = null;
		category = null;
		serie = null;
		value = null;
		domainAxisLabel = null;
		rangeAxisLabel = null;
		plotOrientation = null;
		threeD = false;
	}
	
	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public String getCollection() {
		return collection;
	}

	public void setCollection(String collection) {
		this.collection = collection;
	}

	public String getDomainAxisLabel() {
		return domainAxisLabel;
	}

	public void setDomainAxisLabel(String domainAxisLabel) {
		this.domainAxisLabel = domainAxisLabel;
	}

	public String getPlotOrientation() {
		return plotOrientation;
	}

	public void setPlotOrientation(String plotOrientation) {
		this.plotOrientation = plotOrientation;
	}

	public String getRangeAxisLabel() {
		return rangeAxisLabel;
	}

	public void setRangeAxisLabel(String rangeAxisLabel) {
		this.rangeAxisLabel = rangeAxisLabel;
	}

	public String getSerie() {
		return serie;
	}

	public void setSerie(String serie) {
		this.serie = serie;
	}

	public boolean getThreeD() {
		return threeD;
	}

	public void setThreeD(boolean threeD) {
		this.threeD = threeD;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}
}
