/* 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package org.sf.charttaglibs.charts;

import java.util.Iterator;

import javax.servlet.jsp.JspException;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;

/**
 * Class that builds up a Line Chart (2D & 3D), specifying category, serie, value
 * and plot orentation (optional).
 * @author ftorres
 * @version 1.0
 */
public class LineTag extends CategoryBasedChart {
    
    /**
     * Overrides the initChart method of BaseChartTag
     * @see org.sf.charttaglibs.BaseChartTag#initChart()
     */
    public void initChart() throws JspException {
        if (collection != null) {
			Iterator collIterator = getIterator(collection, null);
			DefaultCategoryDataset dataset = getDataSet();
			chart = null;
			// setting the plot orientation
			PlotOrientation po = null;
			if(plotOrientation != null && plotOrientation.equalsIgnoreCase("HORIZONTAL")){
				po = PlotOrientation.HORIZONTAL;
			}
			else {
				po = PlotOrientation.VERTICAL; // The default value
			}
			//setting up the chart
			if(threeD)
				chart = ChartFactory.createLineChart3D(
			        title, // chart title
					domainAxisLabel, // domain axis label
					rangeAxisLabel, // range axis label
					dataset, // data
					po, // plot orientation
					createLegend, // include legend
					createTooltip, // tooltips?
					createUrl // URLs?
				);
			else
				chart = ChartFactory.createLineChart(
				        title, // chart title
						domainAxisLabel, // domain axis label
						rangeAxisLabel, // range axis label
						dataset, // data
						po, // plot orientation
						createLegend, // include legend
						createTooltip, // tooltips?
						createUrl // URLs?
				);
        }
    }
    
}
