package org.sf.charttaglibs.charts;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import javax.servlet.jsp.JspException;

import org.apache.commons.beanutils.BeanUtils;

import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.util.Rotation;

import org.sf.charttaglibs.BaseChartTag;
import org.sf.charttaglibs.utils.Message;

/**
 * Abstract class theat implement the common method of all pie based chart
 * @author mmohamed |at| dcc |dot| uchile |dot| cl
 */
public abstract class PieBasedChart extends BaseChartTag {

    ///The message resources for this package. 
    private static final Message messages = new Message();
	
    /// The collection of the dataset
	protected String collection = null;
	/// The category name
	protected String category = null;
	/// the value name
	protected String value = null;
	/// the explode position of the pie
	protected int explodePosition = 0;
	/// the explode percent of the pie
	protected float explodePercent = 0;
	/// The chart is 3D
	protected boolean threeD = false;
	/// The direction of rotation of the pie chart
	protected String rotation = null;
	
    /**
     * Get the dataset to be used in the initChart method to build the Chart. 
     * @return DefaultPieDataset dataset
     * @throws JspException - when whether an IllegalAccessException, an InvocationTargetException or a NoSuchMethodException is caught.
     */
	protected DefaultPieDataset getDataSet() throws JspException {
		
		String filename = null;
		ChartRenderingInfo info = null;
		DefaultPieDataset dataset = new DefaultPieDataset();
		if (collection != null) {
			Iterator collIterator = getIterator(collection, null);
			while (collIterator.hasNext()) {
				
				Object bean = collIterator.next();
				Object cat = null;
				Object val = null;
				
				try {
					cat = BeanUtils.getProperty(bean, category);
					if (cat == null) {
						cat = "";
					}
				} catch (IllegalAccessException e) {
					throw new JspException(
							messages.getMessage("getter.access", category, collection));
				} catch (InvocationTargetException e) {
					Throwable t = e.getTargetException();
					throw new JspException(
							messages.getMessage("getter.result", category, t.toString()));
				} catch (NoSuchMethodException e) {
					throw new JspException(
							messages.getMessage("getter.method", category, collection));
				}
				
				try {
					if (value != null) {
						val = BeanUtils.getProperty(bean, value);
					}
					
					if (val == null) {
						val = "";
					}
					dataset.setValue(cat.toString(), new Float(val.toString()));
				} catch (IllegalAccessException e) {
					throw new JspException(
							messages.getMessage("getter.access", cat, collection));
				} catch (InvocationTargetException e) {
					Throwable t = e.getTargetException();
					throw new JspException(
							messages.getMessage("getter.result", cat, t.toString()));
				} catch (NoSuchMethodException e) {
					throw new JspException(
							messages.getMessage("getter.method", cat, collection));
				}
			}
		}
		return dataset;
	}
	
	/**
	 * Set the plot attribute for the pie chart if existe any
	 * @throws JspException
	 */
	protected void setPlotAttribute() throws JspException {
		if(threeD) {
			PiePlot3D plot = (PiePlot3D)chart.getPlot();
			plot.setExplodePercent(getExplodePosition(), getExplodePercent());
			if("ANTICLOCKWISE".equalsIgnoreCase(rotation))
				plot.setDirection(Rotation.ANTICLOCKWISE);
			else
				plot.setDirection(Rotation.CLOCKWISE);
		} else {
			PiePlot plot = (PiePlot)chart.getPlot();
			plot.setExplodePercent(getExplodePosition(), getExplodePercent());
			if("ANTICLOCKWISE".equalsIgnoreCase(rotation))
				plot.setDirection(Rotation.ANTICLOCKWISE);
			else
				plot.setDirection(Rotation.CLOCKWISE);
		}
	}
	
	/**
     * Release any acquired resources.
     */
	public void release() {
		super.release();
		collection = null;
		category = null;
		value = null;
		explodePosition = 0;
		explodePercent = 0;
		threeD = false;
		rotation = null;
	}
	
	// Getters And Setters
	public String getCollection() {
		return collection;
	}
	
	public void setCollection(String collection) {
		this.collection = collection;
	}
	public String getCategory() {
		return category;
	}
	
	public void setCategory(String category) {
		this.category = category;
	}
	
	public String getValue() {
		return value;
	}
	
	public void setValue(String value) {
		this.value = value;
	}

	public boolean getThreeD() {
		return threeD;
	}

	public void setThreeD(boolean threeD) {
		this.threeD = threeD;
	}

	public float getExplodePercent() {
		return explodePercent;
	}

	public void setExplodePercent(float explodedPercent) {
		this.explodePercent = explodedPercent;
	}

	public int getExplodePosition() {
		return explodePosition;
	}

	public void setExplodePosition(int explodedPosition) {
		this.explodePosition = explodedPosition;
	}

	public String getRotation() {
		return rotation;
	}

	public void setRotation(String rotatioon) {
		this.rotation = rotatioon;
	}
}
