/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package org.sf.charttaglibs.utils;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;


/**
 * Utility method for converting Enumeration to an Iterator
 * class.  If you attempt to remove() an Object from the iterator, it will
 * throw an UnsupportedOperationException. Added for use by TagLib so
 * Enumeration can be supported
 *
 * @version $Rev: 164530 $ $Date: 2005-04-25 04:11:07 +0100 (Mon, 25 Apr 2005) $ 
 */

public class IteratorAdapter implements Iterator {
    private Enumeration e;

    public IteratorAdapter(Enumeration e) {
        this.e = e;
    }

    public boolean hasNext() {
        return e.hasMoreElements();
   }

    public Object next() {
        if (!e.hasMoreElements()) {
            throw new NoSuchElementException("IteratorAdaptor.next() has no more elements");
        }
        return e.nextElement();
    }
    public void remove() {
        throw new UnsupportedOperationException("Method IteratorAdaptor.remove() not implemented");
    }
}
